<?php
/*
Plugin Name: Lockdown Mode
Plugin URI: https://www.pluginvulnerabilities.com/lockdown-mode-plugin/
Update URI: false
Description: Locks down WordPress functionalilty.
Author: White Fir Design
Version: 1.0.1
Author URI: https://www.pluginvulnerabilities.com/

Copyright 2024 White Fir Design

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; only version 2 of the License is applicable.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

//Block direct access to the file
if ( !function_exists( 'add_action' ) ) { 
	exit; 
}
define( "LOCKDOWN_MODE_VERSION","1.0.1" );

class Lockdown_Mode_Plugin {

	public function __construct() {
		
		add_action( 'admin_menu', [ $this, 'admin_menu' ] );
		add_filter( 'plugin_action_links_' . plugin_basename(__FILE__), [ $this,  'information_link' ] );

		if ( defined( 'LOCKDOWN_MODE' ) && LOCKDOWN_MODE == true && defined( 'DISALLOW_FILE_EDIT' ) && DISALLOW_FILE_EDIT == true ) {
			//Lockdown option changes
			add_action( 'add_option', [ $this, 'option_change' ], 1 );
			add_action( 'delete_option', [ $this, 'option_change' ], 1 );
			add_action( 'update_option', [ $this, 'option_change' ], 1 );
		
			//Lockdown role change
			add_filter( 'update_user_meta', [ $this, 'role_change' ], 1, 3);

			//Lockdown post deletion
			add_filter( 'pre_trash_post', [ $this, 'post_deletion' ], 1 );
			add_filter( 'pre_delete_post', [ $this, 'post_deletion' ], 1 );
		
			//Lockdown media deletion
			add_filter( 'pre_delete_attachment', [ $this, 'media_deletion' ], 1 );
			//Lockdown user deletion
			add_action( 'delete_user', [ $this, 'user_deletion' ], 1 );
		
			//Lockdown plugin changes
			add_action( 'activate_plugin', [ $this, 'plugin_change' ], 1 );
			add_action( 'deactivate_plugin', [ $this, 'plugin_change' ], 1 );
			add_filter( 'upgrader_pre_download', [ $this, 'plugin_install' ], 1, 3);
			add_action( 'pre_uninstall_plugin', [ $this, 'plugin_change' ], 1 );
			add_action( 'delete_plugin', [ $this, 'plugin_change' ], 1 );
			
			//Lockdown user registration
			add_filter( 'pre_user_login', [ $this, 'user_registration' ], 1);
			add_filter( 'user_registration_email', [ $this, 'user_registration' ], 1);
			add_action( 'register_post', [ $this, 'user_registration' ], 1);
			
			add_action( 'admin_notices', [ $this, 'lockdown_admin_notice'] );	
			
			if ( defined( 'LOCKDOWN_MODE_OUTBOUND_REQUESTS' ) && LOCKDOWN_MODE_OUTBOUND_REQUESTS == true )
				add_filter( 'http_request_timeout', [ $this, 'outbound_requests' ], 1, 2);
			
		}
	}

	public function admin_menu() {
		add_management_page( 'Lockdown Mode', 'Lockdown Mode', 'manage_options', 'lockdown-mode', [ $this, 'admin_page' ]  );
	}

	public function admin_page() {
		?>
		<h1>Lockdown Mode</h1>
		<p>Lockdown mode restricts the following activity:</p>
		<ul style="list-style-type: disc;">
			<li>Changes to sensitive settings (options)</li>
			<li>Media deletion</li>
			<li>Plugin installation and deletion</li>
			<li>Post deletion</li>
			<li>User role changes</li>
			<li>User registration</li>
			<li>Theme installation and deletion</li>
		</ul>
		<p> For those looking to continue to access the functionality, but want to better protect against vulnerability exploitation, our <a href="https://www.pluginvulnerabilities.com/plugin-vulnerabilities-firewall/">Plugin Firewall Plugin</a> restricts most exploitation while allowing legitimate usage.</p>
		<br>
		<p>To enable the lockdown mode, in the wp-config.php file for the website find the line that reads</p>
		 <pre>/* Add any custom values between this line and the "stop editing" line. */":</pre>
		<p>Below that, add the following lines:</p>
<pre>define( 'DISALLOW_FILE_EDIT', true );
define( 'LOCKDOWN_MODE', true);</pre>
		<br>
		<p>You can confirm the lockdown mode is working by trying to deactivate this plugin on the <a href="<?= get_admin_url()."plugins.php"; ?>">Installed Plugins page</a>.</p>
		<br>
		<p>To block outbound requests from the website other than to the website itself and wordpres.org, add this additional line to the wp-config.php file:
		<pre>define( 'LOCKDOWN_MODE_OUTBOUND_REQUESTS', true);</pre>	
		<br>
		<p>To disable the lockdown mode, remove the code above or change the "true" in it to "false".</p>
		<?php
	}

	function information_link( $links ) {
		$information_link = '<a href="'.esc_url(get_admin_url(). "tools.php?page=lockdown-mode").'">Instructions</a>';
		array_unshift( $links, $information_link );
		return $links;
	}

	public function lockdown_admin_notice() {
		?>
		<div class="notice notice-error">
		<p>Lockdown Mode enabled. <a href="<?= esc_url(get_admin_url()."tools.php?page=lockdown-mode") ?>">Disable</a></p>
		</div>
		<?php
	}

	public function media_deletion() {
		wp_die("Action stopped because Lockdown Mode enabled.");
	}

	public function option_change ( $option_name ) {
		$restricted_options = array ( 'active_plugins', 'admin_email', 'blog_public', 'blogdescription', 'blogname', 'comment_moderation', 'comment_registration', 'comments_notify', 'default_role', 'home', 'mailserver_login', 'siteurl', 'stylesheet', 'template', 'users_can_register', 'wp_user_roles', );
		if ( in_array( strtolower( remove_accents ( $option_name ) ) , $restricted_options ) ) {
			wp_die("Action stopped because Lockdown Mode enabled.");
		}
		else
			return;
	}

	public function outbound_requests($timeout_value, $url) {
		$host = parse_url( $url )["host"];
		if( substr( $host, -13 ) != "wordpress.org" && $host != parse_url( get_option( 'siteurl' ) )["host"] )
			wp_die("Action stopped because Lockdown Mode enabled.");
		return $timeout_value;
	}

	public function plugin_change() {
		wp_die("Action stopped because Lockdown Mode enabled.");
	}
	
	public function plugin_install ( $reply, $package, $upgrader ) {
		//Don't block plugin updates
		if ( !isset( $upgrader->strings['up_to_date'] ) )
			wp_die("Action stopped because Lockdown Mode enabled.");
		return $reply;
	}
	
	public function post_deletion() {
		wp_die("Action stopped because Lockdown Mode enabled.");
	}

	public function role_change( $umeta_id, $user_id, $meta_key ) {
		//Capabilites meta_key includes database prefix
		global $wpdb;
		$capabilities_meta_key = $wpdb->base_prefix.'capabilities';
		if ( strtolower( remove_accents ( $meta_key ) ) == $capabilities_meta_key ) {
			wp_die("Action stopped because Lockdown Mode enabled.");
		}
		return $umeta_id;
	}
	
	public function user_deletion() {
		wp_die("Action stopped because Lockdown Mode enabled.");
	}
	
	public function user_registration() {
		wp_die("Action stopped because Lockdown Mode enabled.");
	}
}

new Lockdown_Mode_Plugin();

include('updater.php');